module Gdas
  class CollectionHistory
    include Import['cache.redis']

    NO_TTL = 0
    ONE_DAY_TTL = 86_400
    DISABLED = 'disabled'.freeze
    # TODO: customize length based on job SLA
    LIST_LENGTH = 5

    def fail(customer_id, aws_account_id, collector, exception:)
      key = history_key customer_id, aws_account_id, collector
      res = ::MessagePack.pack(
        success: 0,
        exception: exception.class.name,
        message: exception.message,
        ts: ::Time.now.to_i
      )
      redis.add_to_fixed_size_list key, res, LIST_LENGTH, NO_TTL
    end

    def success(customer_id, aws_account_id, collector, start_ts:, duration:)
      key = history_key customer_id, aws_account_id, collector
      res = ::MessagePack.pack(
        success: 1,
        ts: start_ts.to_i,
        duration: duration
      )
      redis.add_to_fixed_size_list key, res, LIST_LENGTH, NO_TTL
    end

    def last_job(customer_id, aws_account_id, collector)
      key = history_key customer_id, aws_account_id, collector
      job = redis.lrange key, 0, 0
      return nil if job.blank?
      ::MessagePack.unpack(job.first)
    end

    def all_jobs(customer_id, aws_account_id, collector, count = 0)
      key = history_key customer_id, aws_account_id, collector
      count = count.to_i
      len = count.positive? ? count - 1 : -1
      jobs = redis.lrange key, 0, len
      return [] if jobs.blank?
      jobs.map { |job| ::MessagePack.unpack job }
    end

    def disable(customer_id, aws_account_id, collector, reason = nil, ttl = nil)
      key = state_key customer_id, aws_account_id, collector
      res = {
        state: DISABLED,
        ts: ::Time.now.to_i
      }
      if reason
        res[:reason] = reason
      end

      # Default of one day, but can be overridden.
      ttl = ONE_DAY_TTL unless ttl

      redis.add_hash key, res, ttl
    end

    def allow(customer_id, aws_account_id, collector)
      key = state_key customer_id, aws_account_id, collector
      redis.clear key
    end

    def disabled?(customer_id, aws_account_id, collector)
      key = state_key customer_id, aws_account_id, collector
      res = redis.from_hash key
      return false unless res
      res['state'] == DISABLED
    end

    def disabled_reason(customer_id, aws_account_id, collector)
      key = state_key customer_id, aws_account_id, collector
      res = redis.from_hash key
      return nil unless res
      res['reason']
    end

    def allowed?(customer_id, aws_account_id, collector)
      !disabled? customer_id, aws_account_id, collector
    end

    private

    def history_key(customer_id, aws_account_id, collector)
      [
        'collection'.freeze,
        'history'.freeze,
        customer_id,
        aws_account_id,
        collector.service,
        collector.asset
      ].join ':'
    end

    def state_key(customer_id, aws_account_id, collector)
      [
        'collection'.freeze,
        'state'.freeze,
        customer_id,
        aws_account_id,
        collector.service,
        collector.asset
      ].join ':'
    end
  end
end
