# frozen_string_literal: true
module Gdas
  module Helpers
    module RegionHelper
      extend ::ActiveSupport::Concern

      DEFAULT_AWS_REGION_NAME = 'us-east-1'

      # Attempt to extract the region from the given region name. Here are the rules:
      # - If the region name is nil, empty, or 'global' default to 'us-east-1'
      # - If the region name matches a supported region, return it.
      # - If the region name starts with something that looks like a region, try looking up the partial match.
      #
      # Params:
      # - +region_name+:: String. The region name from the superartifact to match against.
      #
      # Returns:
      # - +AwsRegion+ object if there is a match.
      # - +nil+ if the lookup failed.
      def aws_region_from_region_name(region_name)
        # Default value if region is missing or set to global
        if region_name.blank? || region_name == 'global'
          return common_assets.regions_by_name[DEFAULT_AWS_REGION_NAME]
        end

        region = common_assets.regions_by_name[region_name]
        return region if region

        # The region field can also contain local zone information (ex: "us-west-2-lax-1").
        # In that case, try to parse a region from the field and lookup again.
        matches = region_name.match(/\A([a-z]+-[a-z]+-[1-9]+)/) # Does it start with something that looks like a region?
        if matches
          parsed_region = matches[1]
          region = common_assets.regions_by_name[parsed_region]
        end

        region
      end
    end
  end
end
