require 'job_management'

module Gdas::JobMgmt

  # Subclass of TokenLock that:
  #  - Combines traditional unique job locking with token locking
  #  - Specifies what keys lock on for processing jobs token locking: the supported_owners determine
  #     the dimensions the token lock applies to
  #
  # Unique job locking framework taken from SidekiqUniqueJobs::Lock::WhileExecuting.
  # NOTE: When upgrading SidekiqUniqueJobs you must modify this to match the
  #   WhileExecuting class in the version you are upgrading to!
  class DerivedProcessorTokenLock < ::Gdas::JobMgmt::TokenLock
    # Constants declared in SidekiqUniqueJobs (outside of class BaseLock class)
    LOCK_TIMEOUT_KEY          ||= "lock_timeout"
    UNIQUE_DIGEST_KEY         ||= "lock_digest"

    # New constant declared for WhileExecuting subclass
    RUN_SUFFIX                ||= ":RUN"

    # @param [Hash] item the Sidekiq job hash
    # @param [Proc] callback callback to call after unlock
    # @param [Sidekiq::RedisConnection, ConnectionPool] redis_pool the redis connection
    #
    def initialize(item, callback, redis_pool = nil, statsd_p = nil)
      super(item, callback, redis_pool, statsd_p)
      append_unique_key_suffix
    end

    def supported_owners
      ::Gdas::JobMgmt::DerivedProcessorTokenLock.supported_owners
    end

    def self.supported_owners
      ['customer_id']
    end

    def context_from_item(item)
      return {} if item.nil? || item['args'].nil? || item['args'][1].nil?
      item['args'][1][0]
    end

    # Executes in the Sidekiq server process.
    #   These jobs are locked in the server process not from the client
    # @yield to the worker class perform method
    def execute(&block)
      return call_strategy unless locksmith.lock(wait: item[LOCK_TIMEOUT_KEY])
      super
    rescue Exception => ex # rubocop:disable Lint/RescueException
      locksmith.delete!
      raise
    else
      unlock_and_callback
    end

    private

    # This is safe as the base_lock always creates a new digest
    #   The append there for needs to be done every time
    def append_unique_key_suffix
      item[UNIQUE_DIGEST_KEY] = item[UNIQUE_DIGEST_KEY] + RUN_SUFFIX
    end
  end
end
